# -*- coding: utf-8 -*-
"""Test the CrawlModel object."""

import pytest
from parker import crawlmodel
from parker.configloader import load_site_config
from test_client import client_fixture_crawl
from test_page import page_fixture_crawl
from test_crawlpage import crawlpage_fixture
import utils

TEST_CONFIG_NAME = "staples"
TEST_CONFIG = load_site_config(TEST_CONFIG_NAME)
EXPECTED_URI_BASE = "http://www.staples.co.uk/"
uri_list = utils.load_stub_as_json('expecteduris.json')
uri_list.remove(
    "http://www.staples.co.uk/content/static/customerservice"
    "/aboutus/staples_soul.cshtml?icid=ft:sssl"
)
EXPECTED_URIS = set(uri_list)


@pytest.fixture(scope="function")
def crawlmodel_fixture(crawlpage_fixture):
    """Test fixture to ensure correct mocking for parsedpage."""
    return crawlmodel.get_instance(crawlpage_fixture)


def test_get_instance_creates_crawlmodel_object(crawlpage_fixture):
    """Test crawlmodel.get_instance creates a CrawlModel object."""
    test_crawlmodel = crawlmodel.get_instance(crawlpage_fixture)

    assert isinstance(test_crawlmodel, crawlmodel.CrawlModel)


def test_get_instance_raises_typeerror_unexpected_parameter_type():
    """Test crawlmodel.get_instance throws TypeError on unexpected param."""
    with pytest.raises(TypeError):
        test_crawlmodel = crawlmodel.get_instance(None)


def test_load_from_config_populates_model(crawlmodel_fixture):
    """Test crawlmodel.load_from_config populates the model members."""
    test_crawlmodel = crawlmodel_fixture
    test_crawlmodel.load_from_config(TEST_CONFIG)

    assert test_crawlmodel.site == TEST_CONFIG_NAME
    assert test_crawlmodel.uri_base == EXPECTED_URI_BASE
    assert test_crawlmodel.uris_to_crawl == EXPECTED_URIS
    assert not test_crawlmodel.is_consume_page



# -*- coding: utf-8 -*-
"""RedisSet object for Parker."""

import redis
from configloader import load_config

_instances = dict()
_redis_config = load_config("redis")
_redis = redis.StrictRedis(**_redis_config)
_seconds_in_a_day = 60 * 60 * 24
_seconds_in_five_days = 5 * _seconds_in_a_day


def get_instance(key, expire=None):
    """Return an instance of RedisSet."""
    global _instances
    try:
        instance = _instances[key]
    except KeyError:
        instance = RedisSet(
            key,
            _redis,
            expire=expire
        )
        _instances[key] = instance

    return instance


class RedisSet(object):

    """Wrapper object for Redis sets."""

    def __init__(self, key, redis, expire=None):
        """Constructor."""
        self.key = key
        self.redis = redis
        self.expire = expire or _seconds_in_five_days

    def __repr__(self):
        """Return an unambiguous representation."""
        return "%s(%s)" % (self.__class__, self.key)

    def has(self, value):
        """Test set has value."""
        return self.redis.sismember(
            self.key,
            value
        )

    def add(self, value):
        """Add value to set."""
        added = self.redis.sadd(
            self.key,
            value
        )

        if self.redis.scard(self.key) < 2:
            self.redis.expire(self.key, self.expire)

        return added

    def delete(self, value):
        """Delete key from set."""
        return self.redis.srem(
            self.key,
            value
        )

    def destroy(self):
        """Destroy the set."""
        return self.redis.delete(self.key)

    def ttl(self):
        """Return the TTL for the set."""
        return self.redis.ttl(self.key)



# -*- coding: utf-8 -*-
"""ConsumePage object for Parker."""

import urlparse
import page
import parser
import mediafile

_instances = dict()


def get_instance(page_to_consume):
    """Return an instance of ConsumePage."""
    global _instances
    if isinstance(page_to_consume, basestring):
        uri = page_to_consume
        page_to_consume = page.get_instance(uri)
    elif isinstance(page_to_consume, page.Page):
        uri = page_to_consume.uri
    else:
        raise TypeError(
            "get_instance() expects a parker.Page or basestring derivative."
        )

    page_to_consume.fetch()
    parsed_page = parser.parse(page_to_consume)

    try:
        instance = _instances[uri]
    except KeyError:
        instance = ConsumePage(
            parsed_page
        )
        _instances[uri] = instance

    return instance


class ConsumePage(object):

    """A ConsumePage."""

    def __init__(self, parsedpage):
        """Constructor."""
        self.parsedpage = parsedpage
        self.uri = parsedpage.page.uri
        self.hash = parsedpage.page.hash

    def __repr__(self):
        """Return an unambiguous representation."""
        return "%s(%s)" % (self.__class__, self.uri)

    def get_key_value_dict_by_selectors(
        self, key_selector, value_selector, value_sub_selector=None
    ):
        """Return a dictionary of key value data."""
        key_nodes = self.parsedpage.get_nodes_by_selector(key_selector)
        keys = [
            self.parsedpage.get_text_from_node(node)
            for node in key_nodes
        ]

        value_nodes = self.parsedpage.get_nodes_by_selector(value_selector)
        if value_sub_selector is not None:
            vals = [
                [
                    self.parsedpage.get_text_from_node(subnode)
                    for subnode in node.find(value_sub_selector)
                ]
                for node in value_nodes.items()
            ]
        else:
            vals = [
                self.parsedpage.get_text_from_node(node)
                for node in value_nodes
            ]

        return dict(zip(keys, vals))

    def get_crumb_list_by_selector(self, crumb_selector):
        """Return a list of crumbs."""
        return [
            self.parsedpage.get_text_from_node(crumb)
            for crumb in self.parsedpage.get_nodes_by_selector(crumb_selector)
        ]

    def get_media_list_by_selector(
        self, media_selector, media_attribute="src"
    ):
        """Return a list of media."""
        page_url = urlparse.urlparse(self.uri)
        return [
            mediafile.get_instance(
                urlparse.urljoin(
                    "%s://%s" % (
                        page_url.scheme,
                        page_url.netloc
                    ),
                    urlparse.urlparse(
                        media.attrib[media_attribute],
                        scheme="http"
                    ).geturl()
                )
            )
            for media in self.parsedpage.get_nodes_by_selector(media_selector)
        ]

    def get_data_dict_from_config(self, config_dict):
        """Return a dictionary of data inferred from config_dict."""
        return {
            key: self.parsedpage.get_filtered_values_by_selector(
                item_dict['selector'],
                item_dict.get('regex_filter', None),
                item_dict.get('regex_group', 1)
            )
            for key, item_dict in config_dict.iteritems()
            if item_dict.get('selector', None) is not None
        }



# -*- coding: utf-8 -*-
"""Module of utilities for testing."""

import json
import os
import urllib2

STUB_PATH = os.path.join(
    os.environ['PROJECT'],
    'test',
    'stubs'
)


def is_online():
    """Return boolean representing online status."""
    try:
        # Ping google.co.uk IP
        urllib2.urlopen('http://173.194.41.191', timeout=1)
        return True
    except urllib2.URLError:
        pass

    return False


def load_stub_as_string(filename):
    """Load a stub file."""
    filepath = os.path.join(
        STUB_PATH,
        filename
    )

    with open(filepath, "r") as myfile:
        return myfile.read()


def load_stub_as_iterable(filename):
    """Load a stub file as an iterable."""
    filepath = os.path.join(
        STUB_PATH,
        filename
    )

    return open(filepath, "r")


def load_stub_as_json(filename):
    """Load a stub file as JSON."""
    filepath = os.path.join(
        STUB_PATH,
        filename
    )

    return json.load(open(filepath))



# -*- coding: utf-8 -*-
"""Test the worker methods."""

import os.path
import shutil
import pytest
from parker import fileops
from parker.workers import consumer, crawler, killer, get_site_sets
from parker.queues import crawl_q, consume_q
from test_client import client_fixture, client_fixture_crawl
import utils

TEST_URI = "http://www.staples.co.uk/full-strip-stapler/cbs/412852.html"
TEST_URI_CRAWL = "http://www.staples.co.uk/"
TEST_SITE = "staples"
TEST_CLASSIFICATION = "supplies"
TEST_METHOD = "file"
EXPECTED_FILE = os.path.join(
    fileops.DATA_DIR,
    TEST_CLASSIFICATION,
    TEST_SITE,
    TEST_SITE + '.data'
)
EXPECTED_DATA_DICT = {
    'title': 'Full Strip Stapler',
    'sku': 'WW-412852'
}
EXPECTED_MEDIA_FILE = os.path.join(
    fileops.IMG_DIR,
    TEST_CLASSIFICATION,
    TEST_SITE,
    'WW-',
    '412',
    '852',
    'WW-412852_0.jpg'
)
EXPECTED_JOBS = 298
EXPECTED_TTL = 259200


def test_consumer_writes_data_to_file(client_fixture):
    """Test workers.consumer writes correct data to file system."""
    consumer(TEST_SITE, TEST_URI)

    assert os.path.isfile(EXPECTED_MEDIA_FILE)

    assert os.path.exists(EXPECTED_FILE)
    assert os.path.isfile(EXPECTED_FILE)

    line = fileops.get_line_from_file(EXPECTED_FILE).next()

    for key, value in EXPECTED_DATA_DICT.iteritems():
        assert value in line

    os.remove(EXPECTED_FILE)
    shutil.rmtree(os.path.join(
        fileops.IMG_DIR,
        TEST_CLASSIFICATION,
        TEST_SITE
    ))


@pytest.mark.skipif(not utils.is_online(), reason="Currently offline.")
def test_crawler_adds_links_to_crawl_queue(client_fixture_crawl):
    """Test workers.crawler adds links to the crawl queue."""
    killer(TEST_SITE)
    crawler(TEST_SITE, TEST_URI_CRAWL)

    assert len(crawl_q.jobs) == EXPECTED_JOBS
    assert len(consume_q.jobs) == 0

    killer(TEST_SITE)


@pytest.mark.skipif(not utils.is_online(), reason="Currently offline.")
def test_crawler_adds_links_to_consume_queue(client_fixture):
    """Test workers.crawler adds links to the consume queue."""
    killer(TEST_SITE)
    crawler(TEST_SITE, TEST_URI)

    assert len(crawl_q.jobs) == 0
    assert len(consume_q.jobs) == 1

    killer(TEST_SITE)


@pytest.mark.skipif(not utils.is_online(), reason="Currently offline.")
def test_crawler_obtains_expiry_from_config(client_fixture):
    killer(TEST_SITE)
    crawler(TEST_SITE, TEST_URI)
    visited_set, visited_uri_set, consume_set, crawl_set = get_site_sets(
        TEST_SITE
    )

    assert visited_set.ttl() <= EXPECTED_TTL
    assert visited_uri_set.ttl() <= EXPECTED_TTL
    assert consume_set.ttl() <= EXPECTED_TTL
    assert crawl_set.ttl() <= EXPECTED_TTL


# -*- coding: utf-8 -*-
"""Test the configuration loader."""

import pytest
from parker.configloader import load_config, load_site_config

TEST_CONFIG_NAME = 'redis'
TEST_CONFIG_KEY = 'host'
TEST_CONFIG_VALUE = 'localhost'
TEST_SITE_NAME = 'staples'
EXPECTED_SITE_CONFIG = {
    "id": "staples",
    "class": "supplies",
    "tags": ["office", "stationary"],
    "seconds_until_expire": 259200,
    "uri_base": "http://www.staples.co.uk/",
    "uri_start_crawl": "http://www.staples.co.uk/",
    'crawl_uri_filters': ['staples_soul'],
    "consume_selector": "#PageInner .skuPage",
    "unique_field": "sku",
    "specific_data": {
        "title": {
            "selector": "h1",
            "regex_filter": "Staples ([a-zA-Z0-9 ]+)"
        },
        "sku": {
            "selector": ".skuDetails .iNumber",
            "regex_filter": "([a-zA-Z0-9\\-]+)$"
        },
        "colour": {
            "kv_ref": "Colour :"
        }
    },
    "key_value_data": {
        "key_selector": "#divSpecifications dd .l",
        "value_selector": "#divSpecifications dd .r"
    },
    "crumbs": {
        "selector": "#skuBreadCrumbs span[itemprop=title]"
    },
    "media": {
        "selector": ".s7Thumbs img",
        "attribute": "data-zoomimage"
    }
}


def test_load_config_loads_json_into_a_dict():
    """Test 'redis' config loads and that 'host' is 'localhost'."""
    config = load_config(TEST_CONFIG_NAME)

    assert config[TEST_CONFIG_KEY] == TEST_CONFIG_VALUE


def test_load_site_config_loads_json_into_a_dict():
    """Test 'staples' test content is loaded correctly."""
    site_config = load_site_config(TEST_SITE_NAME)

    assert site_config == EXPECTED_SITE_CONFIG



# -*- coding: utf-8 -*-
"""File functions for Parker."""

import json
import os
from configloader import load_config
from stringops import generate_chunks

_config = load_config("parker")

VAR_DIR = _config.get(
    "storage-directory",
    os.path.join(
        os.environ.get('PROJECT'),
        'var'
    )
)
LOG_DIR = os.path.join(VAR_DIR, 'log')
DATA_DIR = os.path.join(VAR_DIR, 'data')
IMG_DIR = os.path.join(VAR_DIR, 'img')


def create_dirs(path):
    """Create all directories in @path."""
    try:
        os.makedirs(path)
    except OSError, error:
        if error.errno != 17:
            raise


def dump_string_to_file(string, filepath):
    """Dump @string as a line to @filepath."""
    create_dirs(
        os.path.dirname(filepath)
    )

    with open(filepath, 'a') as outfile:
        outfile.write(string)
        outfile.write('\n')


def dump_dict_to_file(dictionary, filepath):
    """Dump @dictionary as a line to @filepath."""
    create_dirs(
        os.path.dirname(filepath)
    )

    with open(filepath, 'a') as outfile:
        json.dump(dictionary, outfile)
        outfile.write('\n')


def get_line_from_file(filename):
    """Generator to return a file line by line."""
    for row in open(filename, 'rU'):
        yield row.strip()


def get_chunk_path_from_string(string, chunk=3):
    """Return a chunked path from string."""
    return os.path.join(
        *list(generate_chunks(
            string,
            chunk
        ))
    )



# -*- coding: utf-8 -*-
"""CrawlPage object for Parker."""

import re
import page
import parser
import urlparse

_instances = dict()


def get_instance(page_to_consume):
    """Return an instance of CrawlPage."""
    global _instances
    if isinstance(page_to_consume, basestring):
        uri = page_to_consume
        page_to_consume = page.get_instance(uri)
    elif isinstance(page_to_consume, page.Page):
        uri = page_to_consume.uri
    else:
        raise TypeError(
            "get_instance() expects a parker.Page or basestring derivative."
        )

    page_to_consume.fetch()
    parsed_page = parser.parse(page_to_consume)

    try:
        instance = _instances[uri]
    except KeyError:
        instance = CrawlPage(
            parsed_page
        )
        _instances[uri] = instance

    return instance


class CrawlPage(object):

    """A CrawlPage."""

    def __init__(self, parsedpage):
        """Constructor."""
        self.parsedpage = parsedpage
        self.uri = parsedpage.page.uri
        self.hash = parsedpage.page.hash

    def __repr__(self):
        """Return an unambiguous representation."""
        return "%s(%s)" % (self.__class__, self.uri)

    def get_uris(self, base_uri, filter_list=None):
        """Return a set of internal URIs."""
        return {
            re.sub(r'^/', base_uri, link.attrib['href'])
            for link in self.parsedpage.get_nodes_by_selector('a')
            if 'href' in link.attrib and (
                link.attrib['href'].startswith(base_uri) or
                link.attrib['href'].startswith('/')
            ) and
            not is_uri_to_be_filtered(link.attrib['href'], filter_list)
        }

    def has_selector(self, consume_selector):
        """Test if page has selector."""
        return bool(self.parsedpage.get_nodes_by_selector(consume_selector))


def is_uri_to_be_filtered(uri, filter_list):
    """Test whether @uri should be filtered by @filter_list."""
    match = False
    if filter_list:
        for uri_filter in filter_list:
            if re.search(uri_filter, uri, flags=re.IGNORECASE):
                match = True
        return match



# -*- coding: utf-8 -*-
"""Test the ConsumeModel object."""

import os
import shutil
import pytest
from parker import consumemodel, fileops
from parker.configloader import load_site_config
from parker.mediafile import MediaFile
from test_client import client_fixture
from test_page import page_fixture
from test_consumepage import consumepage_fixture

TEST_URI = "http://www.staples.co.uk/full-strip-stapler/cbs/412852.html"
TEST_CONFIG_NAME = "staples"
TEST_CONFIG = load_site_config(TEST_CONFIG_NAME)
TEST_FILE = "/tmp/test.data"
TEST_PATH = "/tmp"
EXPECTED_KV_DICT = {
    u'Staple Compatibility :': u'26/06/2014',
    u'Brands :': u'Staples',
    u'Stapling Capacity (sheets) :': u'20 sheets',
    u'Quantity :': u'1'
}
EXPECTED_CRUMB_LIST = [
    u'Pens | Tape | Desk Supplies',
    u'Desktop Stationery | Clips | Accessories',
    u'Staplers'
]
EXPECTED_MEDIA_LIST_URLS = [
    'http://www.staples.co.uk/content/images/product/uk_412852_1_xnl.jpg'
]
EXPECTED_DATA_DICT = {
    'title': 'Full Strip Stapler',
    'sku': 'WW-412852',
    'colour': 'Black'
}
EXPECTED_UNIQUE_FIELD = 'WW-412852'
EXPECTED_MEDIA_PATH = "/tmp/staples/WW-/412/852"
EXPECTED_MEDIA_FILE = "/tmp/staples/WW-/412/852/WW-412852_0.jpg"


@pytest.fixture(scope="function")
def consumemodel_fixture(consumepage_fixture):
    """Test fixture to ensure correct mocking for parsedpage."""
    return consumemodel.get_instance(consumepage_fixture)


def test_get_instance_creates_consumemodel_object(consumepage_fixture):
    """Test consumemodel.get_instance creates a ConsumeModel object."""
    test_consumemodel = consumemodel.get_instance(consumepage_fixture)

    assert isinstance(test_consumemodel, consumemodel.ConsumeModel)


def test_get_instance_raises_typeerror_unexpected_parameter_type():
    """Test consumemodel.get_instance throws TypeError on unexpected param."""
    with pytest.raises(TypeError):
        test_consumemodel = consumemodel.get_instance(None)


def test_load_from_config_populates_model(consumemodel_fixture):
    """Test consumemodel.load_from_config populates the model members."""
    test_consumemodel = consumemodel_fixture
    test_consumemodel.load_from_config(TEST_CONFIG)

    assert test_consumemodel.site == TEST_CONFIG_NAME
    assert test_consumemodel.data_dict == EXPECTED_DATA_DICT
    assert test_consumemodel.key_value_dict == EXPECTED_KV_DICT
    assert test_consumemodel.crumb_list == EXPECTED_CRUMB_LIST
    assert test_consumemodel.unique_field == EXPECTED_UNIQUE_FIELD

    for mediafile in test_consumemodel.media_list:
        assert isinstance(mediafile, MediaFile)
        assert mediafile.uri in EXPECTED_MEDIA_LIST_URLS



# -*- coding: utf-8 -*-
"""Test the CrawlPage object."""

import pytest
from parker import parser, crawlpage, parsedpage
from test_client import client_fixture_crawl, client_fixture
from test_page import page_fixture_crawl, page_fixture
import utils

TEST_URI = "http://www.staples.co.uk/"
TEST_CONSUME_SELECTOR = "#PageInner .skuPage"
EXPECTED_URI_COUNT = 300
EXPECTED_URIS = set(utils.load_stub_as_json('expecteduris.json'))


@pytest.fixture(scope="function")
def crawlpage_fixture(page_fixture_crawl):
    """Test fixture to ensure correct mocking for crawlpage."""
    return crawlpage.get_instance(
        page_fixture_crawl
    )


def test_get_instance_creates_crawlpage_object(page_fixture_crawl):
    """Test crawlpage.get_instance creates a CrawlPage object."""
    test_crawlpage = crawlpage.get_instance(
        page_fixture_crawl
    )
    expected_repr = "<class 'parker.crawlpage.CrawlPage'>(%s)" % (
        TEST_URI
    )

    assert isinstance(test_crawlpage, crawlpage.CrawlPage)
    assert isinstance(test_crawlpage.parsedpage, parsedpage.ParsedPage)
    assert test_crawlpage.__repr__() == expected_repr


def test_get_instance_raises_typeerror_unexpected_parameter_type():
    """Test crawlpage.get_instance throws TypeError on unexpected param."""
    with pytest.raises(TypeError):
        test_crawlpage = crawlpage.get_instance(None)


def test_get_uris_returns_list_of_internal_uris(crawlpage_fixture):
    """Test crawlpage.get_uris returns a set of internal URIs."""
    test_crawlpage = crawlpage_fixture
    uris = test_crawlpage.get_uris(TEST_URI)

    assert isinstance(uris, set)
    assert len(uris) == EXPECTED_URI_COUNT
    assert uris == EXPECTED_URIS


def test_has_selector_returns_false_if_not(crawlpage_fixture):
    """Test crawlpage.has_selector returns false if selector not in page."""
    test_crawlpage = crawlpage_fixture

    assert not test_crawlpage.has_selector(TEST_CONSUME_SELECTOR)


def test_has_selector_returns_true_if_has(page_fixture):
    """Test crawlpage.has_selector returns true if selector in page."""
    test_crawlpage = crawlpage.get_instance(
        page_fixture
    )

    assert test_crawlpage.has_selector(TEST_CONSUME_SELECTOR)



# -*- coding: utf-8 -*-
"""Test the Store object."""

import pytest
from mock import MagicMock, call
from parker import store
from test_mediafile import mediafile_fixture, client_fixture

TEST_BUCKET = 'test_bucket.test.parker.python'
TEST_FILENAME = 'BIG/FAT/TEST_FILE_0'
TEST_DICT = {
    "foo": "bar",
    "fuzzy": "duck",
    "big": {
        "data": "google",
        "butts": "sir mixalot"
    },
    "list": [
        "barney",
        "fred"
    ]
}
EXPECTED_FILENAME = '/tmp/BIG/FAT/TEST_FILE_0'
EXPECTED_JSON = '{"fuzzy": "duck", "foo": "bar", "list": ["barney", "fred"], "big": {"data": "google", "butts": "sir mixalot"}}'


@pytest.fixture(scope="function")
def s3store_fixture(monkeypatch):
    """Test fixture to ensure correct mocking for mediafile."""
    monkeypatch.setattr(
        store.boto,
        'connect_s3',
        MagicMock()
    )
    test_store = store.get_s3store_instance(
        bucket=TEST_BUCKET
    )
    monkeypatch.setattr(
        store,
        'Key',
        MagicMock()
    )

    return test_store


def test_store_media_sets_contents_from_temp_filename(
    s3store_fixture, mediafile_fixture
):
    s3store = s3store_fixture
    s3store.store_media(
        TEST_FILENAME,
        mediafile_fixture
    )

    kall = call().set_contents_from_filename(
        EXPECTED_FILENAME + '.jpg'
    )

    assert store.Key.mock_calls[1] == kall.call_list()[1]


def test_store_json_sets_contents_as_json_dumps(
    s3store_fixture
):
    s3store = s3store_fixture
    s3store.store_json(
        TEST_FILENAME,
        TEST_DICT
    )

    kall = call().set_contents_from_string(
        EXPECTED_JSON,
        headers={'Content-Type': 'application/json; charset=UTF-8'},
        replace=True
    )

    assert store.Key.mock_calls[1] == kall.call_list()[1]



# -*- coding: utf-8 -*-
"""Parser abstraction that wraps PyQuery for Parker."""

from pyquery import PyQuery
import page
import parsedpage

_parsed = dict()


def parse(page_to_parse):
    """Return a parse of page.content. Wraps PyQuery."""
    global _parsed
    if not isinstance(page_to_parse, page.Page):
        raise TypeError("parser.parse requires a parker.Page object.")

    if page_to_parse.content is None:
        raise ValueError("parser.parse requires a fetched parker.Page object.")

    try:
        parsed = _parsed[page_to_parse]
    except KeyError:
        parsed = parsedpage.ParsedPage(
            page=page_to_parse,
            parsed=PyQuery(page_to_parse.content, parser='html')
        )
        _parsed[page_to_parse] = parsed

    return parsed



# -*- coding: utf-8 -*-
"""Model object for consumed pages in Parker."""

from datetime import datetime
import os.path
import consumepage
import fileops

_instances = dict()


def get_instance(page_to_consume):
    """Return an instance of ConsumeModel."""
    global _instances
    if isinstance(page_to_consume, basestring):
        uri = page_to_consume
        page_to_consume = consumepage.get_instance(uri)
    elif isinstance(page_to_consume, consumepage.ConsumePage):
        uri = page_to_consume.uri
    else:
        raise TypeError(
            "get_instance() expects a parker.ConsumePage "
            "or basestring derivative."
        )

    try:
        instance = _instances[uri]
    except KeyError:
        instance = ConsumeModel(page_to_consume)
        _instances[uri] = instance

    return instance


class ConsumeModel(object):

    """Model representing a consumed page."""

    def __init__(self, consumepage):
        """Constructor."""
        self.consumepage = consumepage
        self.uri = consumepage.uri
        self.hash = consumepage.hash
        self.classification = None
        self.tags = None
        self.unique_field = None
        self.site = None
        self.data_dict = None
        self.key_value_dict = None
        self.crumb_list = None
        self.media_list = None

    def load_from_config(self, config):
        """Load model from passed configuration."""
        self.site = config.get("id", False)
        self.classification = config.get("class", False)
        self.tags = config.get("tags", False)
        self._load_key_value(
            config.get("key_value_data", False)
        )
        self._load_data(
            config.get("specific_data", False)
        )
        self._load_crumb(
            config.get("crumbs", False)
        )
        self._load_media_list(
            config.get("media", False)
        )
        self.unique_field = self.data_dict.get(
            config.get("unique_field", False),
            False
        )

    def get_dict(self):
        """Return a dictionary of the object primed for dumping."""
        data = self.data_dict.copy()

        data.update({
            "class": self.classification,
            "tags": self.tags,
            "key_value_data": self.key_value_dict,
            "crumbs": self.crumb_list if len(self.crumb_list) > 0 else None,
            "media": [
                mediafile.filename
                if mediafile.filename is not None
                else mediafile.uri
                for mediafile in self.media_list
            ] if len(self.media_list) > 0 else None,
            "uri": self.uri,
            "dumped": datetime.utcnow().isoformat(),
        })

        return data

    def _load_data(self, data_config):
        self.data_dict = self.consumepage.get_data_dict_from_config(
            data_config
        ) if data_config else None
        self._post_process_data(data_config)

    def _load_key_value(self, key_value_config):
        self.key_value_dict = self.consumepage.get_key_value_dict_by_selectors(
            key_value_config["key_selector"],
            key_value_config["value_selector"],
            key_value_config.get("sub_selector", None)
        ) if (
            key_value_config
            and key_value_config.get("key_selector", False)
            and key_value_config.get("value_selector", False)
        ) else None

    def _load_crumb(self, crumb_config):
        self.crumb_list = self.consumepage.get_crumb_list_by_selector(
            crumb_config["selector"]
        ) if (
            crumb_config
            and crumb_config.get("selector", False)
        ) else None

    def _load_media_list(self, media_config):
        self.media_list = self.consumepage.get_media_list_by_selector(
            media_config["selector"],
            media_config["attribute"],
        ) if (
            media_config
            and media_config.get("selector", False)
            and media_config.get("attribute", False)
        ) else None

    def _post_process_data(self, config):
        for key, item in config.iteritems():
            kv_ref = item.get('kv_ref', False)
            value = self.key_value_dict.get(kv_ref, False)
            if kv_ref and value:
                self.data_dict[key] = self.key_value_dict.pop(kv_ref)



# -*- coding: utf-8 -*-
"""Parker.

A web spider for collecting specific data
across a set of configured sites.
"""

import os
from setuptools import setup, find_packages
from pip.req import parse_requirements


def read(*names):
    """Read files for description."""
    values = dict()
    for name in names:
        filename = name + '.rst'
        if os.path.isfile(filename):
            fd = open(filename)
            value = fd.read()
            fd.close()
        else:
            value = ''
        values[name] = value
    return values


long_description = """
%(README)s

Changes
====================

%(CHANGES)s

""" % read('README', 'CHANGES')

version_file = open('VERSION')
version = version_file.read().strip()

install_reqs = parse_requirements('requirements.txt')
reqs = [str(ir.req) for ir in install_reqs]

setup(
    name='Parker',
    version=version,
    url='http://github.com/nefarioustim/parker/',
    license='GPL v3',
    author='Tim Huegdon',
    author_email='tim@nefariousdesigns.co.uk',
    description='A web spider for collecting specific data across a set of '
                'configured sites',
    long_description=long_description,
    install_requires=reqs,
    packages=['parker'],
    scripts=[
        'bin/parker-clean',
        'bin/parker-crawl',
        'bin/parker-config'
    ],
    include_package_data=True,
    zip_safe=False
)



# -*- coding: utf-8 -*-
"""MediaFile object for Parker."""

import client

_instances = dict()
_content_type_map = {
    'image/jpeg': 'jpg',
    'image/png': 'png',
    'image/gif': 'gif'
}


def get_instance(uri):
    """Return an instance of MediaFile."""
    global _instances
    try:
        instance = _instances[uri]
    except KeyError:
        instance = MediaFile(
            uri,
            client.get_instance()
        )
        _instances[uri] = instance

    return instance


class MediaFile(object):

    """A MediaFile, as fetched from a client."""

    def __init__(self, uri, client):
        """Constructor."""
        self.uri = uri
        self.client = client
        self.filename = None
        self.fileext = None

    def __repr__(self):
        """Return an unambiguous representation."""
        return "%s(%s)" % (self.__class__, self.uri)

    def fetch_to_file(self, filename):
        """Stream the MediaFile to the filesystem."""
        self.filename = filename
        stream = self.client.get_iter_content(self.uri)
        content_type = self.client.response_headers['content-type']

        if content_type in _content_type_map:
            self.fileext = _content_type_map[content_type]
            self.filename = "%s.%s" % (self.filename, self.fileext)

        with open(self.filename, 'wb') as file_handle:
            for chunk in stream:
                file_handle.write(chunk)
